#!/opt/local/bin/python


# DETAILS
# - Author: StJohn Piano
# - Date: 2018-10-05
# - Description: This script generates a standard Bitcoin address from a private key. 


# ENVIRONMENTS
# - Tested on:
# -- Python 2.7.13 running on Mac OS X 10.6.8 (Snow Leopard).


# NOTES
#
# - I define a "standard" address to be an uncompressed single-signature Pay-To-Public-Key-Hash (P2PKH) address. 
# - This script was developed under Python 2.7.13 running on Mac OS X 10.6.8. 
#
# This script takes as input one of the following two options:
# -- 1) a private key in raw byte form e.g.
# 		"hello_world"
# -- 2) a private key in hex form e.g. 
#		"2851cc22ea3fdfc71ae6282f54b801178f1e8ead64d9c916b211f109d80c0dba"
#
# - This script outputs:
# -- The private key in raw bytes, if this is the chosen input type. 
# -- The private key in hex bytes.
# -- The private key in WIF form. WIF = Wallet Import Format. 
# -- The Bitcoin address. 
# 
# - Dependencies: 
# -- bitcoin_functions.py: A set of functions written by StJohn Piano. 


# INSTRUCTIONS:
#
# 1) Choose the type of private key.
# - Set private_key_type to "raw_bytes" to use the value of private_key_raw_bytes as the private key when this script is run.
# - Set private_key_type to "hex_bytes" to use the value of private_key_hex_bytes as the private key when this script is run.
#
# 2) Set the value of the private key input. 
# - private_key_raw_bytes must be a sequence of printable ASCII characters. 
# - private_key_hex_bytes must be a sequence of hex characters. Hex characters in input variable values must be lowercase. 

# Notes: 
# - If an input is not used, it can be set to an empty string "".
# - The printable ASCII characters (newline, tab, and space are not shown), in order, are:
# ! " # $ % & ' ( ) * + , - . / 0 1 2 3 4 5 6 7 8 9 : ; < = > ? @ A B C D E F G H I J K L M N O P Q R S T U V W X Y Z [ \ ] ^ _ ` a b c d e f g h i j k l m n o p q r s t u v w x y z { | } ~
# -- Using a starting index of zero for the ASCII character set, the indices of the printable characters are:
# --- 9 (tab)
# --- 10 (newline) 
# --- 32 (space)
# --- [33 to 126 inclusive] (the other printable characters)
# - hex characters:
# -- 0 1 2 3 4 5 6 7 8 9 a b c d e f
# - When used, a private key must be at least 1 byte long and at most 32 bytes long.  
# -- For private_key_raw_bytes this is between 1 and 32 ASCII characters.
# -- For private_key_hex_bytes this is between 2 and 64 hex characters (1 byte is represented by 2 hex characters).



# DEPENDENCIES
import bitcoin_functions as bf


def main():
	
	
	##### START CONTROLS
	# Input variables are all strings. Unused input variables can be set to the empty string {""}. 
	# Possible values for private_key_type:
	# ["raw_bytes", "hex_bytes"]
	private_key_type = "raw_bytes"
	private_key_raw_bytes = "roadside_picnic"
	# Hex characters in input variable values must be lowercase. 
	private_key_hex_bytes = "2851cc22ea3fdfc71ae6282f54b801178f1e8ead64d9c916b211f109d80c0dba"
	##### END CONTROLS
	
	
	##### START SETTINGS
	
	private_key_type_options = ["raw_bytes", "hex_bytes"]
	
	##### END SETTINGS
	
	
	# preparation
	bf.validate_item_in_list(private_key_type, private_key_type_options)
	
	
	print "\n### START GENERATION OF BITCOIN ADDRESS\n"
	
	if private_key_type == "raw_bytes":
	
		bf.validate_string(private_key_raw_bytes)
	
		input = private_key_raw_bytes
		print "Private key (raw bytes): %s" % input
		
		bf.validate_bitcoin_private_key_bytes(input)
		
		input = bf.bytes_to_hex(input)
		print "Private key (hex bytes): %s" % input
	
		input = bf.left_pad_hex_to_length_x(input, 32)
		print "Private key (32 hex bytes): %s" % input
	
	elif private_key_type == "hex_bytes":
		
		bf.validate_hex(private_key_hex_bytes)
		
		input = private_key_hex_bytes
		
		print "Private key (hex bytes): %s" % input
		
		input = bf.left_pad_hex_to_length_x(input, 32)
		print "Private key (32 hex bytes): %s" % input
	
	bf.validate_bitcoin_private_key_hex(input)
	
	private_key_wif = bf.private_key_hex_to_wif(input)
	print "Private key (WIF): %s" % private_key_wif
	
	bitcoin_address = bf.private_key_hex_to_bitcoin_address(input)
	print "Bitcoin address: %s " % bitcoin_address
	
	print "\n### END GENERATION OF BITCOIN ADDRESS\n"
	

if __name__ == '__main__': main()